/*
  cfg.c: control and configure - common independent functions
  Andriy Gritsenko (C) 2003
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef DEBUG
#include <assert.h>
#endif

#include "mp_msg.h"

#include "cfg.h"

/* find subtree arg in tree conf */
cfg_tree_t *cfg_find(cfg_tree_t *conf, char *arg)
{
  size_t l;

  for ( ; conf ; conf = conf->next ) {
    if (conf->opt->name == NULL)
      continue;
    l = strlen(conf->opt->name);
    if (l == 0)
      continue;
    // check for full matching first
    if (strcasecmp(conf->opt->name,arg) == 0)
      break;
    // check if it's dummy - descent into it without ':'
    else if (conf->opt->type == CONF_TYPE_DUMMY) {
      cfg_tree_t *sc = cfg_find(conf->subtree,arg);
      if (sc)
        return sc;
    // check for opt:subopt syntax
    } else if (CONF_TYPE_HAS_CHILD(conf->opt->type) &&
               !strncasecmp(conf->opt->name,arg,l) && arg[l] == ':') {
      return cfg_find(conf->subtree,&arg[l+1]);
    // check for wildcards
    } else if (CONF_TYPE_ALLOW_WILDCARD(conf->opt->type) &&
               (conf->opt->name[l-1] == '*')) {
      if (strncasecmp(conf->opt->name,arg,l-1) == 0)
        break;
    }
  }
  return conf;
}

/* register option opt in (sub)tree config */
static int cfg_add_option(cfg_tree_t **config, cfg_t *opt)
{
  cfg_tree_t *co;

#ifdef DEBUG
  assert(config != NULL);
  assert(opt != NULL);
#endif

  // check if option already exist - just ignore it with warning
  if (cfg_find(*config, opt->name)) {
    //mp_msg();
    return 0;
  }

  // allocate a new entry for this option
  co = (cfg_tree_t*)calloc(1,sizeof(cfg_tree_t));
  co->opt = opt;

  // option with childs => add them (note: CHOOSE may be limited)
  if (CONF_TYPE_HAS_CHILD(opt->type))
    cfg_register(&co->subtree, (cfg_t *)opt->priv,
		 (opt->flags & CONF_MAX) ? opt->max : 0);

  // insert it at the end - preserve order always
  while (*config) config = &(*config)->next;
  (*config) = co;
  return 1;
}

/* register null-terminated list of options in config (max n items, 0 - unlim)
 * returns number of succesfully registered options
 */
int cfg_register(cfg_tree_t **config, cfg_t *list, int n)
{
  int i;

#ifdef DEBUG
  assert(config != NULL);
#endif

  if (!list) // if empty subconfig don't try descent into it
    return 0;

  for (i = 0; list[i].name != NULL && (n == 0 || i < n); i++)
    cfg_add_option(config, &list[i]);
  return i;
}

/* unregister null-terminated list of options in config
 * returns number of succesfully unregistered options
 */
int cfg_unregister(cfg_tree_t **config, cfg_t *list)
{
  int i, n = 0;

  for (i = 0; list[i].name != NULL; i++) {
    cfg_tree_t **co;

    for (co = config; *co; co = &(*co)->next)
      if ((*co)->opt == &list[i])
	break;
    if (*co) {
      cfg_tree_t *del = *co;

      *co = del->next;
      free(del);
      n++;
    } //else // option not found - warn
      //mp_msg();
  }
  return n;
}
