/*
  cfg.h: all UI/parsers API
  Andriy Gritsenko (C) 2003
  Note: don't forget to increase PLUGIN_VERSION_MINOR if you added something,
  or PLUGIN_VERSION_MAJOR if you inserted, or reordered, or changed syntax
 */

#ifndef __CFG_H
#define __CFG_H 1

// ----------------------------------------------------------------------------
//  plugin level -- API for modules
//  --- note: this part of it has to be moved in plugin.h later
/* module type, control module only currently */
#define PLUGIN_TYPE_CTL		0

/* version control variables - 0...255 */
#define PLUGIN_VERSION_MAJOR	0
#define PLUGIN_VERSION_MINOR	0

struct pl_instance_s;
struct cfg_tree_s;

/* common for all modules - all *_info_t may be gone in the future
  that structure is ModuleInfo structure for dlsym()
  it has to have version[] as first member! */
typedef struct {
  unsigned char version[2]; // { PLUGIN_VERSION_MAJOR, PLUGIN_VERSION_MINOR }
  int type;
  char *name;
  char *desc; // short description
  char *maintainer;
  char *author;
  char *comment;
  int (*open)(struct pl_instance_s *pli); // plugin's open function
  struct pl_func_s *f; // module type specific functions
  struct cfg_s *opts; // options pointed to exchange buffer
} pl_info_t;

/* I bet it's not ready yet until unification */
typedef struct pl_instance_s {
  pl_info_t *info;
  struct pl_instance_s *prev; // previous plugin in the chain
  struct pl_func_s *f; // just copy from info to decrease depth
  struct pl_priv_s *priv; // module specific private data
} pl_instance_t;

/* register module, returns 0 on success
   conf is config leaf of that module (should not be NULL!)
   preopen is module type specific function to fill pli->f structure */
int pl_register_plugin(pl_info_t *pl, struct cfg_tree_s *conf, int (*preopen)(pl_instance_t *pli,pl_instance_t *last));
/* get module instance by name and insert it in list,
   returns no-NULL on success */
pl_instance_t *pl_get_plugin(pl_instance_t *last, int type, char *name);
/* open module, returns 0 on success */
int pl_open_plugin(pl_instance_t *pli);

/* functions for exchange of options - unified versions, for example for VF
    ((vf_instance_t*)pl->priv)->control(pl->priv,VFCTRL_GET_PARAMS,NULL)
int pl_get_params(pl_instance_t *pl);
int pl_set_params(pl_instance_t *pl);
*/
//  --- end of plugin.h part ---

//#include "plugin.h"

// ----------------------------------------------------------------------------
//  cfg_t level -- all common API

/* type (menu field)	->p		->min	->max	->priv */
typedef enum {
// flag (checkbox)	int*		reset	set	-
  CONF_TYPE_FLAG=0,
// integer (slider)	int*		min	max	-
  CONF_TYPE_INT,
// float (slider)	float*		min	max	-
  CONF_TYPE_FLOAT,
// position (slider)	off_t*		min	max	-
  CONF_TYPE_POSITION,
// string (input box)	char**		min	max	-
  CONF_TYPE_STRING,
// string as list - must be obsoleted now!
  CONF_TYPE_STRING_LIST,
// function		cfg_func*	-	-	revert-cfg_func*
  CONF_TYPE_FUNC,
  CONF_TYPE_FUNC_PARAM,
  CONF_TYPE_FUNC_FULL,
// text (none)		char**		-	-	-
  CONF_TYPE_PRINT,
/*
all below this have
  ->priv as uoption_t*
*/
// subconfig (dialog)	-		-	-	uoption_t*
  CONF_TYPE_SUBCONFIG,
// choose list		char**		-	length	uoption_t*
  CONF_TYPE_CHOOSE,
// select list		char***		-	-	uoption_t*
  CONF_TYPE_LIST,
// plugin list		pl_instance_t**	-	type	uoption_t*
  CONF_TYPE_CHAIN,
// config group (sep.)	-		-	-	uoption_t*
  CONF_TYPE_DUMMY
} cfg_type;

#define CONF_TYPE_HAS_CHILD(x)		(x>=CONF_TYPE_SUBCONFIG)
#define CONF_TYPE_ALLOW_WILDCARD(x)	(x==CONF_TYPE_LIST||x==CONF_TYPE_CHAIN||x==CONF_TYPE_FUNC_FULL)

/* main structure - former struct config */
typedef struct cfg_s {
  char *name;
  void *p; /* variable pointer - see above */
  cfg_type type;
  unsigned int flags;
  float min, max; /* range - see above */
  void *priv; /* type specific - see above */
  char *desc; /* short description */
} cfg_t;

/* parser errors */
#define ERR_NOT_AN_OPTION	-1
#define ERR_MISSING_PARAM	-2
#define ERR_OUT_OF_RANGE	-3
#define ERR_FUNC_ERR		-4

/* flags for options */
#define CONF_MIN		(1<<0)
#define CONF_MAX		(1<<1)
#define CONF_RANGE		(CONF_MIN|CONF_MAX)
#define CONF_NOCFG		(1<<2)
#define CONF_NOCMD		(1<<3)
// This option is global : it won't be saved and only the command line
// parser will set it when it's parsed (ie. it won't be set later)
// e.g options : -v, -quiet
#define CONF_GLOBAL		(1<<4)
// Do not save this option : it won't be saved on config save
// or by UI on context change
#define CONF_NOSAVE		(1<<5)
// This subconfig has unnamed parameters list (i.e. 0.3:-1:-1 for example)
// It looks like a regular subconfig but different in parsing a bit
#define CONF_SEQUENT		(1<<6)
// It used for lists to reverse order of arguments in char*** or pl_instance_t**
#define CONF_REVERSE		(1<<7)


// ----------------------------------------------------------------------------
//  pl_func_s level -- functions API for all control modules

/* note: any function may be NULL - check on call! */
struct pl_func_s {
  int (*parse)(pl_instance_t *, struct cfg_tree_s *, int, char **); // some types need that
  int (*uninit)(pl_instance_t *, struct cfg_tree_s *); // drop unparsed and revert opts
  char *(*get_unparsed)(pl_instance_t *, struct cfg_tree_s *, char *); // get value by name
  int (*save)(struct cfg_tree_s *, FILE *); // save changed values into file
};


// ----------------------------------------------------------------------------
//  cfg_tree_t level -- common independent API

/* module independent tree of options */
typedef struct cfg_tree_s {
  struct cfg_tree_s *next;
  cfg_t *opt; // not allocated, only reference to existing option
  struct cfg_tree_s *subtree;
  unsigned int flags; // currently it only tell if the option was changed
} cfg_tree_t;

// for CONF_TYPE_FUNC_FULL
typedef int (*cfg_func_arg_param_t)(cfg_tree_t *, char *, char *);
// for CONF_TYPE_FUNC_PARAM
typedef int (*cfg_func_param_t)(cfg_tree_t *, char *);
// for CONF_TYPE_FUNC
typedef int (*cfg_func_t)(cfg_tree_t *);

/* (un)register null-terminated list of options in config (max n items, 0 unlim)
 * returns number of (un)registered options
 */
int cfg_register(cfg_tree_t **config, cfg_t *list, int n);
int cfg_unregister(cfg_tree_t **config, cfg_t *list);

/* find subtree name in tree config */
cfg_tree_t *cfg_find(cfg_tree_t *config, char *name);

#endif /* __CFG_H */
